import gtk
import gtk.gdk as gdk
import gobject

class BindingDialog(object):
    def __init__(self, glade, dialog, parent, actionTypes):
        self.dialog = dialog

        renderer = gtk.CellRendererText()

        self.actionTypes = actionTypes
        
        self.glade = glade
        
        self.setupActionList(renderer)
        self.fillActionList()

        self.eventTypeMenu = self.glade.get_widget('eventtype_optionmenu')
        self.fillEventTypeMenu()

        self.glade.get_widget('button_cancel').connect('clicked',
            lambda widget: self.dialog.hide())


        self.glade.get_widget('button_grab').connect(
            'clicked', self.on_button_grab_clicked)

        self.glade.get_widget('button_ok').connect(
            'clicked', self.on_button_ok_clicked)

        self.dialog.set_transient_for(parent)        
        self.dialog.show()

        self.keyReleaseHandler = None

        self.currentBinding = {'ok': False,
                               'eventType': 'KeyPress' }

        self.mode = 'new'


    def fillEventTypeMenu(self):
        menu = gtk.Menu()
        
        for label, value in self.eventTypes():
            if label is None or value is None:
                menu_item = gtk.SeparatorMenuItem()
            else:
                menu_item = gtk.MenuItem(label)
                menu_item.connect('activate',
                                  self.on_eventtype_optionmenu_activate, value)
                
            menu_item.show()
            menu.append(menu_item)

        self.eventTypeMenu.set_menu(menu)

    def eventTypes(self):
        return [('KeyPress', 'Key press'),
                ('KeyRelease', 'Key release'),
                ('ButtonPress', 'Mouse button press'),
                ('ButtonPress', 'Mouse button release'),
                ('DoubleCLick', 'Mouse double click'),
                ('EnterNotify', 'Pointer entering a window'),
                ('LeaveNotify', 'Pointer leaving a window'),
                ('MapRequest',  'New window')]

    def on_eventtype_optionmenu_activate(self, widget, event):
        self.currentBinding['eventType'] = self.currentEventType()

    def on_grab_keyrelease(self, widget, event):
        accelerators = [
            (gdk.MOD4_MASK, 'Super'),            
            (gdk.CONTROL_MASK, 'Control'),
            (gdk.MOD1_MASK, 'Alt'),            
            (gdk.SHIFT_MASK, 'Shift')]

        acceleratorString = ''

        self.currentBinding['notModifier'] = []
        
        self.currentBinding['modifier'] = []
        
        for mask, name in accelerators:
            if event.state & mask:
                acceleratorString += '<%s>' % name
                self.currentBinding['modifier'].append(name)

        self.glade.get_widget('label_key').set_text(
            acceleratorString + ' ' + gdk.keyval_name(event.keyval))

        self.currentBinding['name'] = gdk.keyval_name(event.keyval)
        self.currentBinding['ok']   = True

        widget.disconnect(self.keyReleaseHandler)
        gdk.pointer_ungrab()
        gdk.keyboard_ungrab()

    def on_grab_buttonrelease(self, widget, event):
        accelerators = [
            (gdk.MOD4_MASK, 'Super'),            
            (gdk.CONTROL_MASK, 'Control'),
            (gdk.MOD1_MASK, 'Alt'),            
            (gdk.SHIFT_MASK, 'Shift')]

        acceleratorString = ''

        for mask, name in accelerators:
            if event.state & mask:
                acceleratorString += '<%s>' % name

        self.glade.get_widget('label_key').set_text(
            acceleratorString + ' button %d' % event.button)

        widget.disconnect(self.mouseReleaseHandler)
        gdk.pointer_ungrab()
        gdk.keyboard_ungrab()

    def currentEventType(self):
        return self.eventTypes()[self.eventTypeMenu.get_history()][0]

    def keyboardEventSelected(self):
        return self.currentEventType() in ('KeyPress', 'KeyRelease')

    def mouseEventSelected(self):
        return self.currentEventType() in ('ButtonPress', 'ButtonRelease', 'DoubleClick')

    def on_button_grab_clicked(self, widget):
        print "GRABBING!"

        if self.keyboardEventSelected():
            gdk.pointer_grab(widget.window, False, gdk.BUTTON_PRESS_MASK)            
            gdk.keyboard_grab(widget.window, False)
            self.glade.get_widget('label_key').set_text('Press a key')
            self.keyReleaseHandler = widget.connect('key_release_event',
                                                    self.on_grab_keyrelease)
        elif self.mouseEventSelected():
            self.glade.get_widget('label_key').set_text(
                'Press a mouse button (optionally with Ctrl, Alt, etc.) here')
            
            self.mouseReleaseHandler = \
                                     self.glade.get_widget('label_key').connect(
                'button-press-event',  self.on_grab_buttonrelease)
        else:
            print "*** WARNING: Grab not handled"

    def on_button_ok_clicked(self, widget):
        if self.mode == 'new':
            if self.currentBinding['ok']:
                eventType = self.currentBinding['eventType']
                
                params = {'modifier':    self.currentBinding['modifier'],
                          'notModifier': self.currentBinding['notModifier'],
                          'name':        self.currentBinding['name']}
                
                self.glade.appendBinding((eventType,
                                          params,
                                          self.currentAction))
                
                self.glade.refreshActionList()
                self.dialog.hide()
            else:
                ### TODO: Notify user he can't save it yet
                pass

    def on_actionList_change(self, widget):
        selected = []
        widget.selected_foreach(lambda model, path, ite:
                                selected.append(path))
        self.currentAction = self.actionList.get_model()[selected[0]][0]
        
    def setupActionList(self, renderer):
        self.actionList = self.glade.get_widget('actions_list')

        if not self.actionList.get_column(0):            
            self.actionList.append_column(gtk.TreeViewColumn('Name',
                                                             renderer, text=0))

        selection = self.actionList.get_selection()
        selection.connect("changed", self.on_actionList_change)

    def fillActionList(self):        
        actions = {
            'window': ['Shade', 'Close', 'Maximize', 'Minimize',
                       'MaximizeVertically',
                       'MinimizeVertically'],
            'screen': ['GoToDesktop',
                       'NextDesktop',
                       'PreviousDesktop']}
            

        model = gtk.ListStore(str)

        for actionType in self.actionTypes:
            for action in actions[actionType]:
                model.append((action,))

        self.actionList.set_model(model)

