import pygtk

pygtk.require('2.0')

import gtk
import gtk.gdk as gdk
import gtk.glade
import gobject

class ValidationError(Exception):
    def __init__(self, message, widget):
        Exception.__init__(self, 'Validation error %s in %s' % \
                           (message, widget))
        
        self.message = message
        self.widget  = widget

class Setting(object):
    """Abstract base object for settings"""
    def __init__(self, config, label, name):
        self.config = config
        self.label  = label
        self.name   = name
        
    def createWidget(self):
        raise NotImplementedError

    def createLabel(self):
        label = gtk.Label(self.label + ':')
        label.set_alignment(0.0, 0.5)
        label.set_use_underline(True)
        return label

    def load(self):
        raise NotImplemented()

    def save(self):
        raise NotImplemented()
    
    def value(self):
        return self.config.settings.get(self.name, '')

    def setValue(self, value):
        self.config.settings[self.name] = value
    
class SimpleSetting(Setting):
    """Simple settings like keyboardMenuNavigation(), no params"""
    def createWidget(self):
        self.widget = gtk.CheckButton(self.label)

    def createLabel(self):
        return None
        
    def save(self):
        if self.widget.get_active():
            self.setValue(self.name)
        else:
            self.setValue('')

    def load(self):
        self.widget.set_active(self.value() != '')

class ComboSetting(Setting):
    """Selection among a fixed set"""
    def createWidget(self):
        optionMenu = gtk.OptionMenu()
        
        menu = gtk.Menu()
        
        for label, value in self.choices():
            menu_item = gtk.MenuItem(label)
            menu_item.show()
            menu.append(menu_item)

        optionMenu.set_menu(menu)

        self.widget = optionMenu

    def save(self):
        self.setValue(self.choices()[self.widget.get_history()][1])

    def load(self):
        values = [value for label, value in self.choices()]

        text = self.value()

        if text not in values: text = values[0]
        
        self.widget.set_history(values.index(text))
        
    def choices(self):
        raise NotImplementedError

class FocusSetting(ComboSetting):
    """Click/sloppy/mouse focus"""
    def choices(self):
        return [('Sloppy focus', 'sloppyFocus'),
                ('Click focus', 'clickFocus'),
                ('Mouse focus', 'mouseFocus')]

class ModifierSetting(ComboSetting):
    """Setting for *withModifier macros. Offers choice between modifiers"""
    def choices(self):
        return [('Control', 'Control'),
                ('Alt', 'Alt'),
                ('Shift', 'Shift'),                
                ('Super (Windows Key)', 'Super')]
                
class ButtonSetting(Setting):
    def createWidget(self):
        hbox = gtk.HBox(spacing=12)

        self.checkboxes = []

        buttons = ['Left', 'Middle', 'Right']

        for i, button in zip(range(len(buttons)), buttons):
            checkbox = gtk.CheckButton(button)            
            self.checkboxes.append(checkbox)
            
            hbox.pack_start(checkbox, expand=False, fill=False)
            checkbox.show()

        self.widget = hbox

    def load(self):
        pass

    def save(self):
        pass


class TwoComboSetting(Setting):
    """Selection among a fixed set"""
    def createWidget(self):
        hbox = gtk.HBox(spacing=12)

        self.optionMenu  = self._createOptionMenu(self.choices())
        self.optionMenu2 = self._createOptionMenu(self.choices2())

        label = gtk.Label('+')
        
        hbox.pack_start(self.optionMenu,  False, True)
        hbox.pack_start(label)
        hbox.pack_start(self.optionMenu2, False, True)

        label.show()
        self.optionMenu.show()
        self.optionMenu2.show()

        self.widget = hbox

    def _createOptionMenu(self, choices):
        optionMenu = gtk.OptionMenu()
        
        menu = gtk.Menu()
        
        for label, value in choices:
            menu_item = gtk.MenuItem(label)
            menu_item.show()
            menu.append(menu_item)
        
        optionMenu.set_menu(menu)
        
        return optionMenu

    def widgetValues(self):
        return self.choices()[self.optionMenu.get_history()][1], \
               self.choices2()[self.optionMenu2.get_history()][1]

    def setWidgetValues(self, value, value2):
        values  = [value for label, value in self.choices()]
        values2 = [value for label, value in self.choices2()]
        
        if text  not in values:  text  = values[0]
        if text2 not in values2: text2 = values2[0]
        
        self.optionMenu.set_history(values.index(text))
        self.optionMenu2.set_history(values2.index(text2))        

    def save(self):
        raise NotImplementedError

    def load(self):
        raise NotImplementedError
        
    def choices(self):
        raise NotImplementedError
    
    def choices2(self):
        raise NotImplementedError


class ModifierButtonSetting(TwoComboSetting):
    def choices(self):
        return [('Control', 'Control'),
                ('Alt', 'Alt'),
                ('Shift', 'Shift'),                
                ('Super', 'Super')]
    
    def choices2(self):
        return [('Left button', '1'),
                ('Middle button', '2'),
                ('Right button', '3')]

    def save(self):
        modifier, button = self.widgetValues()
        self.setValue('modifier=%s,button=%s' % (modifier, button))

    def load(self):
        if self.value() == '':
            return
            
        modifierStr, buttonStr = self.value().split(',')
        
        modifier = modifierStr.split('=')[1]
        button   = buttonStr.split('=')[1]

        self.setWidgetValues(modifier, button)
    
class DesktopKeysSetting(ComboSetting):
    def choices(self):
        return [('Ctrl+Alt+#', 'modifier=Ctrl+Alt-AltGr, keys=%d'),
                ('Alt+F#', 'modifier=Alt-AltGr, keys=F%d'),
                ('Keypad', 'keys=KP_%d')]
        
class NumericSetting(Setting):
    """Numeric entry for things like window snapping distance"""
    def createWidget(self):
        self.widget = gtk.Entry()
    
    def save(self):
        value = self.widget.get_text()
        
        if value == '':
            self.setValue('')
        else:                
            try:
                value = int(value)
            except ValueError:
                raise ValidationError('Could not convert to integer',
                                      self.widget)

            self.setValue(str(value))
            
    def load(self):
        self.widget.set_text(self.value())
